@@ -1,40 +1,59 @@
 # !/usr/bin/python3
-import configparser
-import io
 import logging
 from argparse import ArgumentParser
 from collections import deque
 from time import sleep
 
-from mppsolar.io import get_port
+import yaml
+
 from mppsolar.libs.mqttbroker import MqttBroker
 from mppsolar.outputs import output_results
+from mppsolar.ports import get_port
 from mppsolar.protocols import get_protocol
 from mppsolar.version import __version__  # noqa: F401
 
+# from mppsolar.io import get_port
+
+
 # Set-up logger
 log = logging.getLogger("")
 FORMAT = "%(asctime)-15s:%(levelname)s:%(module)s:%(funcName)s@%(lineno)d: %(message)s"
 logging.basicConfig(format=FORMAT)
 
+
+class ConfigError(Exception):
+    pass
+
+
 sample_config = """
-    [CONFIG]
-    port = /dev/ttyUSB0
-    porttype = test
-    portbaud = 2400
-    protocol = PI30
-    mqttbroker_name = localhost
-    mqttbroker_port = 1883
-    mqttbroker_user =
-    mqttbroker_pass =
-    split_token = ,
-    commands = QPIGS,QPIRI
-    outputs = screen,mqtt
-    tag = sample
-    filter =
-    command_topic = test/command_topic
-    command_pause = 5
-    """
+  port:
+    path: /dev/ttyUSB0
+    type: test
+    baud: 2400
+  protocol: PI30
+  mqttbroker:
+    name: null
+    port: 1883
+    user: null
+    pass: null
+  adhoc_commands:
+    topic: test/command_topic
+    outputs:
+      - name: screen
+      - name: mqtt
+  commands:
+    - command: QPIGS
+      gap: 10s
+      outputs:
+      - name: screen
+      - name: mqtt
+    - command: QPIRI
+      gap: 1m
+      outputs:
+      - name: screen
+        tag: testtag
+        filter: volt
+"""
 
 ADHOC_COMMANDS = deque([])
 SPLIT_TOKEN = ","
@@ -55,9 +74,9 @@ def main():
         "--configFile",
         nargs="?",
         type=str,
-        help="Full location of config file (default /etc/mpp-solar/powermon.conf)",
-        const=None,
-        default="/etc/mpp-solar/powermon.conf",
+        help="Full location of config file",
+        const="/etc/mpp-solar/powermon.yml",
+        default=None,
     )
     parser.add_argument("-v", "--version", action="store_true", help="Display the version")
     parser.add_argument(
@@ -103,85 +122,89 @@ def main():
     # Build configuration from defaults, config file and command line overrides
     log.info(f"Using config file:{args.configFile}")
     # build config - start with defaults
-    config = configparser.ConfigParser()
-    config.read_string(sample_config)
+    config = yaml.safe_load(sample_config)
     # build config - update with details from config file
     if args.configFile is not None:
-        config.read(args.configFile)
+        with open("args.configFile", "r") as stream:
+            try:
+                config.update(yaml.safe_load(stream))
+            except yaml.YAMLError as exc:
+                print(exc)
     # build config - override with any command line arguments
     # TODO: command line overrides
 
     # if generateConfigFile is true then print config out
     if args.generateConfigFile:
-        # FIXME must be a better way
-        with io.StringIO() as ss:
-            config.write(ss)
-            ss.seek(0)  # rewind
-            print(ss.read())
+        print("# yaml config for powermon")
+        print("# default location /etc/mpp-solar/powermon.yml")
+        print(yaml.dump(config))
         return
 
     # debug dump config
-    for section in config.sections():
-        log.debug(f"config section [{section}]")
-        for key in config[section]:
-            print(f"{key} = {config[section][key]}")
-
-    # split token
-    SPLIT_TOKEN = config.get("CONFIG", "split_token")
+    log.debug(config)
 
     # Build mqtt broker
+    # TODO disable if not defined
     mqtt_broker = MqttBroker(
-        name=config.get("CONFIG", "mqttbroker_name"),
-        port=config.getint("CONFIG", "mqttbroker_port"),
-        username=config.get("CONFIG", "mqttbroker_user"),
-        password=config.get("CONFIG", "mqttbroker_pass"),
+        name=config["mqttbroker"]["name"],
+        port=config["mqttbroker"]["port"],
+        username=config["mqttbroker"]["user"],
+        password=config["mqttbroker"]["pass"],
     )
     log.debug(mqtt_broker)
+
     # sub to command topic
+    # TODO disable if not defined
+    # TODO disable if mqttbroker is null
     mqtt_broker.connect()
-    mqtt_broker.subscribe(config.get("CONFIG", "command_topic"), mqtt_callback)
+    mqtt_broker.subscribe(config["adhoc_commands"]["topic"], mqtt_callback)
     # connect to mqtt
     mqtt_broker.start()
 
     # get port
-    port = get_port(
-        port=config.get("CONFIG", "port"),
-        porttype=config.get("CONFIG", "porttype"),
-        baud=config.get("CONFIG", "portbaud"),
-    )
+    portconfig = config["port"].copy()
+    log.debug("portconfig", portconfig)
+    port = get_port(portconfig)
+    # port = get_port(porttype=porttype)
+    if not port:
+        log.error(f"No port for config '{portconfig}' found")
+        raise ConfigError(f"No port for config '{portconfig}' found")
 
     # get protocol handler
-    protocol = get_protocol(protocol=config.get("CONFIG", "protocol"))
+    protocol = get_protocol(protocol=config["protocol"])
 
     loop = True
-
     try:
         # connect to port
         port.connect()
         while loop:
             # loop through command list
-            for command in config.get("CONFIG", "commands").split(SPLIT_TOKEN):
+            for command in config["commands"]:
                 # process any adhoc commands first
-                print(f"{ADHOC_COMMANDS}")
+                log.debug(f"adhoc command list: {ADHOC_COMMANDS}")
                 while len(ADHOC_COMMANDS) > 0:
                     adhoc_command = ADHOC_COMMANDS.popleft().decode()  # FIXME: decode to str #
                     log.info(f"Processing command: {adhoc_command}")
                     results = port.process_command(command=adhoc_command, protocol=protocol)
                     log.debug(f"results {results}")
                     # send to output processor(s)
-                    output_results(results=results, config=config, mqtt_broker=mqtt_broker)
+                    # TODO sort outputs
+                    output_results(
+                        results=results, outputs=config["adhoc_commands"], mqtt_broker=mqtt_broker
+                    )
                 # process 'normal' commands
                 log.info(f"Processing command: {command}")
-                results = port.process_command(command=command, protocol=protocol)
+                results = port.process_command(command=command["command"], protocol=protocol)
                 log.debug(f"results {results}")
                 # send to output processor(s)
-                output_results(results=results, config=config, mqtt_broker=mqtt_broker)
+                output_results(results=results, outputs=command, mqtt_broker=mqtt_broker)
                 # pause
-                pause_time = config.getint("CONFIG", "command_pause")
+                pause_time = config["command_pause"]
                 log.debug(f"Sleeping for {pause_time}secs")
-            sleep(pause_time)
             if args.once:
                 loop = False
+            else:
+                sleep(pause_time)
     except KeyboardInterrupt:
         print("KeyboardInterrupt")
     finally:
